#include "basic_creatures.h"
#include "Object.h"
#include "Dungeon.h"

#include <stdio.h>
#include <Alert.h>

// PLAYER

Player::Player( Dungeon * dung )
:	Creature("Player", dung)
{
	INT.base = (random() % 3) + 3; INT.curr = INT.base;
	STR.base = (random() % 3) + 3; STR.curr = STR.base;
	HP.base = 20 + random() % 20; HP.curr = HP.base;

	skills.Add( new Skill("Hand to hand",50) );
	skills.Add( new Skill("1H Sword",50) );
}

Player::~Player()
{
}

void
Player::Attack( Creature * target )
{
	char text[256];
	
	Object * weapon = FindEquiped( RIGHT_HAND );
	
	if ( weapon )
	{ // attack with weapon
		if ( TestSkill( weapon->skill ) )
		{ // hit
			weapon->Use( target, this );
		} else
		{ // miss
			sprintf(text,"%s misses %s.",name,target->name);
			game_message( pos, text );
		}
	
	} else
	{ // attack bare hand
		if ( TestSkill("Hand to hand") )
		{
			damage dmg;
			dmg.source = this;
			dmg.amount = random() % STR.curr + 1;
		
			sprintf(text,"%s punches %s for %d dmg.",name,target->name,dmg.amount);
			game_message( pos, text );

			target->Damage( dmg );
		} else
		{
			sprintf(text,"%s misses %s.",name,target->name);
			game_message( pos, text );
		}
	}

}

PlayerInfo::PlayerInfo()
{
	bmp[0].file = "/add-ons/gfx/warrior/warrior";
	bmp[1].file = NULL;
	bitmaps = bmp;
	name = "Player";
	flags = PLAYABLE;
}

Creature *
PlayerInfo::Create( Dungeon * dung )
{
	Player * player = new Player( dung );
	player->bitmap = bmp[0].id;
	
	return player;
}

// ORC

Orc::Orc( Dungeon * dung)
:	Creature("orc", dung)
{
	INT.base = 4; INT.curr = 4;
	STR.base = 7; STR.curr = 7;
	HP.base = 6; HP.curr = 6;

	skills.Add( new Skill("Sword", 40) );
	
	if ( (random() % 5) == 0 )
	{ // every fifth orc has a health potion (approx)
		Object * obj = dungeon->CreateObject("health potion");
		if (obj)
		{
			inventory.Add( obj->id );
		}
	}
}

Orc::~Orc()
{
}

void
Orc::AI()
{
	Creature * player = dungeon->FindCreature(1);
	if (player && _dist(pos,player->pos) < 8 && dungeon->CastRay(pos, player->pos, 0) )
	{ // player in sight [max sight 8 tiles], go for it!
		if ( FindPath( player->pos ) )
			FollowPath();
	} else
	{ // player not in sight
		if ( HasPath() )
		{ // we have a path, follow it!
			FollowPath();
		} else
		{ // wander aimlessly
			int new_x = pos.x + (random() % 3) - 1;
			int new_y = pos.y + (random() % 3) - 1;
			dungeon->MoveCreature( id, pos, Pos(new_x,new_y) );
		}
	}
}

void
Orc::Death()
{
	Creature::Death();
}

void
Orc::Attack( Creature * target )
{
	char text[256];

	if ( strcmp( target->name, name ) == 0 )
		return; // don't attack fellow orcs
		
	if ( TestSkill("Sword") )
	{
		damage dmg;
		dmg.source = this;
		dmg.amount = random() % 4 + 3;
		
		sprintf(text,"%s hits %s for %d dmg.",name,target->name,dmg.amount);
		game_message( pos, text );
		target->Damage( dmg );
	} else {
		sprintf(text,"%s misses %s.",name,target->name);
		game_message( pos, text );
	}
}

OrcInfo::OrcInfo()
{
	bmp[0].file = "/add-ons/gfx/orc/orc";
	bmp[1].file = NULL;
	bitmaps = bmp;
//	bitmap = "/add-ons/gfx/basic_orc";
	name = "orc";
}

Creature *
OrcInfo::Create( Dungeon * dung )
{
	Orc * orc = new Orc( dung );
	orc->bitmap = bmp[0].id;
	
	return orc;
}

bool
OrcInfo::ShouldPlace( int level )
{
	return (random() % 3) == 0;
}

// RAT

Rat::Rat( Dungeon * dung)
:	Creature("rat", dung),
	irritated(false)
{
	INT.base = 2; INT.curr = 2;
	STR.base = 4; STR.curr = 4;
	HP.base = 4; HP.curr = 4;

	skills.Add( new Skill("Claw", 50) );
}

Rat::~Rat()
{
}

void
Rat::AI()
{
	if ( !irritated )
	{ // wander aimlessly
		int new_x = pos.x + (random() % 3) - 1;
		int new_y = pos.y + (random() % 3) - 1;
		dungeon->MoveCreature( id, pos, Pos(new_x,new_y) );
		return;
	}

	Creature * trg = dungeon->FindCreature(target);
	if (trg && dungeon->CastRay(pos, trg->pos, 0) )
	{ // target in sight, go for it!
		if ( FindPath( trg->pos ) )
			FollowPath();
	} else
	{ // target not in sight
		if ( HasPath() )
		{ // we have a path, follow it!
			FollowPath();
		} else
		{ // wander aimlessly
			int new_x = pos.x + (random() % 3) - 1;
			int new_y = pos.y + (random() % 3) - 1;
			dungeon->MoveCreature( id, pos, Pos(new_x,new_y) );
		}
	}
}

void
Rat::Death()
{
	Creature::Death();
}

void
Rat::Attack( Creature * target )
{
	char text[256];

	if ( strcmp( target->name, name ) == 0 )
		return; // don't attack fellow orcs
		
	if ( TestSkill("Claw") )
	{
		damage dmg;
		dmg.source = this;
		dmg.amount = random() % 3 + 1;
		
		sprintf(text,"%s claws %s for %d dmg.",name,target->name,dmg.amount);
		game_message( pos, text );
		target->Damage( dmg );
	} else {
		sprintf(text,"%s misses %s.",name,target->name);
		game_message( pos, text );
	}
}

void
Rat::Damage( damage dmg )
{
	if ( !irritated )
	{
		char text[256];
		sprintf(text, "rat turns to attack %s!", dmg.source->name);
		game_message( pos, text, be_bold_font, (rgb_color){255,200,0,255} );
	}
	
	irritated = true;
	target = dmg.source->id;
	
	Creature::Damage( dmg );
}

RatInfo::RatInfo()
{
	bmp[0].file = "/add-ons/gfx/white_rat/white_rat";
	bmp[1].file = NULL;
	bitmaps = bmp;
//	bitmap = "/add-ons/gfx/white_rat";
	name = "rat";
}

Creature *
RatInfo::Create( Dungeon * dung )
{
	Rat * rat = new Rat( dung );
	rat->bitmap = bmp[0].id;
	
	return rat;
}

bool
RatInfo::ShouldPlace( int level )
{
	return true;
}

// BASIC BUILDER

BasicBuilder::BasicBuilder()
:	CreatureBuilder( "BasicBuilder" )
{
	info_original[0] = &player_info;
	info_original[1] = &orc_info;
	info_original[2] = &rat_info;
	info_original[3] = NULL;
	info = info_original;

	text = "BasicBuilder";
}

Creature *
BasicBuilder::Create( Dungeon * dung, const char * name )
{
	return NULL;
}

// EXPORT

CreatureBuilder *
CreateBuilder()
{
	return new BasicBuilder;
}
