#ifndef MISC_H
#define MISC_H

#include <Messenger.h>
#include <Font.h>

extern BMessenger game_messenger; // in MainWindow.cpp
enum {
	ROGUE_MSG = 'rgMS'
};

enum  direction {
	UP = 0, UP_RIGHT, RIGHT, DOWN_RIGHT, DOWN, DOWN_LEFT, LEFT, UP_LEFT,
	ALL // full circle, used for LOS
};

enum {
	black_bitmap = 1,
	grey_bitmap,
	wall_bitmap,
	floor_bitmap,
	door_bitmap
};

enum {
	WALL		= 0x00000001,
	FLOOR		= 0x00000002,
	CREATURE	= 0x00000004,
	OBJECT		= 0x00000008,
	DOOR		= 0x00000010,
	EXPLORED	= 0x00000020, // play has visited this tile, display as 'shaded'
	VISIBLE		= 0x00000040, // player can see this tile
	
	INVALID_ID = 0, // this means "if (tile.creature) { display_creature(..) };"
	
	TILE_WIDTH	= 32,	// for graphics
	TILE_HEIGHT = 32
};

class Pos
{
	public:
		Pos();
		Pos(int,int);
		Pos(const Pos &);
		
		bool operator == (const Pos &);
		bool operator != (const Pos &);
		
		Pos operator + ( direction );
		
		int x,y;
};

int _round( float );

class FloatPos
{
	public:
		FloatPos();
		FloatPos(float,float);
		FloatPos(const Pos &); // add 0.49 to each coordinate
		FloatPos(const FloatPos &);
		
		operator Pos ();
		
		bool operator == (const FloatPos &);
		
		const FloatPos & operator += ( const FloatPos & );
		
		float x,y;
};

float _dist( Pos, Pos );

float _fabs( float );

extern rgb_color std_txt;

void game_message( Pos, const char *, BFont font = be_plain_font, rgb_color col = std_txt );

typedef struct {
	const char	* file;
	uint32		id;
} bitmap_info;

#endif
